<?php
/**
 * @file
 * Bulk creation of time slots (simplified form).
 */

/**
 * Form constructor: bulk creation of time slots.
 */
function conference_schedule_timeslots_bulk_form($form, &$form_state) {
  drupal_set_title(t('Add time slots in bulk'));

  $default = cvget('conference_schedule_timeslots_default_timeslots');

  $form['timeslots'] = [
    '#type' => 'textarea',
    '#title' => t('Time slots'),
    '#description' => t('Enter one time slot per line, in the format <code>HH:mm-HH:mm</code>, where “HH:mm” is the time in 24-hour format. The provided time slots will be created for each day of the conference.'),
    '#rows' => 25,
    '#required' => TRUE,
    '#default_value' => $default,
  ];

  $form['actions'] = ['#type' => 'actions'];
  $form['actions']['submit'] = [
    '#type' => 'submit',
    '#value' => t('Create time slots'),
  ];

  return $form;
}

/**
 * Validation handler for conference_schedule_timeslots_bulk_form().
 */
function conference_schedule_timeslots_bulk_form_validate($form, &$form_state) {

}

/**
 * Submit handler for conference_schedule_timeslots_bulk_form().
 *
 * @todo Make lines 61–72 the default handling of conference days in conference_get_days().
 */
function conference_schedule_timeslots_bulk_form_submit($form, &$form_state) {
  $timeslots = array_filter(explode("\n", $form_state['values']['timeslots']));
  $data = conference_schedule_get_scheduling_data();
  $existing_timeslots = $data['conference_timeslots'];
  array_walk($existing_timeslots, function(&$timeslot) {
    $timeslot = "{$timeslot['timeslot']['value']}-{$timeslot['timeslot']['value2']}";
  });
  $keys = ['start', 'end'];

  array_walk($timeslots, function(&$timeslot) use (&$keys) {
    $timeslot = array_combine($keys, array_filter(explode('-', trim($timeslot))));
  });

  $site_timezone = date_default_timezone_get();
  $timezone = conference_date_timezone();

  cvset('conference_schedule_timeslots_default_timeslots', $form_state['values']['timeslots']);

  $days = conference_get_days();

  array_walk($days, function (&$day) use ($timezone, $site_timezone) {
    $date = new DateTime();
    $date->setTimestamp($day['day']);
    $utc = $day['day'] + $date->getOffset();

    date_default_timezone_set($timezone);

    $date = new DateTime();
    $date->setTimestamp($utc);
    $day = strtotime('midnight', $utc - $date->getOffset());
    date_default_timezone_set($site_timezone);
  });

  global $user;
  date_default_timezone_set($timezone);

  foreach ($timeslots as $timeslot) {
    foreach ($days as $day) {
      $start = strtotime($timeslot['start'], $day);
      $end = strtotime($timeslot['end'], $day);

      if (!in_array("{$start}-{$end}", $existing_timeslots)) {
        $node = entity_create('node', [
          'type' => 'conference_timeslot',
          'created' => REQUEST_TIME,
          'changed' => REQUEST_TIME,
          'uid' => $user->uid,
        ]);
        $w = entity_metadata_wrapper('node', $node);
        $w->conference_reference = conference_current();
        $w->conference_timeslot = [
          'value' => $start,
          'value2' => $end,
          'timezone' => $timezone,
        ];
        $w->save();
      }
    }
  }

  date_default_timezone_set($site_timezone);
  $form_state['redirect'] = 'admin/conference/settings/schedule/timeslots';
}
